@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2001,2002,2003 Nikos Mavroyanopoulos
+ *  Copyright (C) 2001,2002,2003 Nikos Mavroyanopoulos
  *
- * This file is part of GNUTLS.
+ *  This file is part of GNUTLS.
  *
  *  The GNUTLS library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public   
@@ -37,9 +37,7 @@
 #include <gnutls_algorithms.h>
 #include <gnutls_global.h>
 #include <gnutls_record.h>
-#include <x509_verify.h>
 #include <gnutls_sig.h>
-#include <x509_extensions.h>
 #include <gnutls_state.h>
 #include <gnutls_pk.h>
 #include <gnutls_x509.h>
@@ -47,14 +45,15 @@
 #include "debug.h"
 
 static int _gnutls_server_find_cert_list_index(gnutls_session session,
-					gnutls_pk_algorithm requested_algo);
+					       gnutls_pk_algorithm
+					       requested_algo);
 
 /* Copies data from a internal certificate struct (gnutls_cert) to 
  * exported certificate struct (CERTIFICATE_AUTH_INFO)
  */
 static
-int _gnutls_copy_certificate_auth_info(CERTIFICATE_AUTH_INFO info,
-				       gnutls_cert * cert, int ncerts)
+int _gnutls_copy_certificate_auth_info(CERTIFICATE_AUTH_INFO info, 
+       gnutls_cert * cert, int ncerts)
 {
 	/* Copy peer's information to AUTH_INFO
 	 */
@@ -65,7 +64,7 @@ int _gnutls_copy_certificate_auth_info(CERTIFICATE_AUTH_INFO info,
 		info->ncerts = 0;
 		return 0;
 	}
-
+	
 	info->raw_certificate_list =
 	    gnutls_calloc(1, sizeof(gnutls_datum) * ncerts);
 	if (info->raw_certificate_list == NULL) {
@@ -77,9 +76,9 @@ int _gnutls_copy_certificate_auth_info(CERTIFICATE_AUTH_INFO info,
 		if (cert->raw.size > 0) {
 			ret =
 			    _gnutls_set_datum(&info->
-					     raw_certificate_list[i],
-					     cert[i].raw.data,
-					     cert[i].raw.size);
+					      raw_certificate_list[i],
+					      cert[i].raw.data,
+					      cert[i].raw.size);
 			if (ret < 0) {
 				gnutls_assert();
 				goto clear;
@@ -102,58 +101,16 @@ int _gnutls_copy_certificate_auth_info(CERTIFICATE_AUTH_INFO info,
 }
 
 
-/* Returns the issuer's Distinguished name in odn, of the certificate 
- * specified in cert.
- */
-int _gnutls_find_dn(gnutls_datum * odn, gnutls_cert * cert)
-{
-	ASN1_TYPE dn;
-	int len, result;
-	int start, end;
-
-	if ((result=_gnutls_asn1_create_element
-	    (_gnutls_get_pkix(), "PKIX1.Certificate", &dn,
-	     "dn")) != ASN1_SUCCESS) {
-		gnutls_assert();
-		return _gnutls_asn2err(result);
-	}
-
-	result = asn1_der_decoding(&dn, cert->raw.data, cert->raw.size, NULL);
-	if (result != ASN1_SUCCESS) {
-		/* couldn't decode DER */
-		gnutls_assert();
-		asn1_delete_structure(&dn);
-		return _gnutls_asn2err(result);
-	}
-
-	result = asn1_der_decoding_startEnd(dn, cert->raw.data, cert->raw.size,
-					"dn.tbsCertificate.issuer", &start,
-					&end);
-
-	if (result != ASN1_SUCCESS) {
-		/* couldn't decode DER */
-		gnutls_assert();
-		asn1_delete_structure(&dn);
-		return _gnutls_asn2err(result);
-	}
-	asn1_delete_structure(&dn);
-
-	len = end - start + 1;
-
-	odn->size = len;
-	odn->data = &cert->raw.data[start];
-
-	return 0;
-}
 
 
 /* returns 0 if the algo_to-check exists in the pk_algos list,
  * -1 otherwise.
  */
 inline
-    static int _gnutls_check_pk_algo_in_list(gnutls_pk_algorithm * pk_algos,
-					     int pk_algos_length,
-					     gnutls_pk_algorithm algo_to_check)
+    static int _gnutls_check_pk_algo_in_list(gnutls_pk_algorithm *
+					     pk_algos, int pk_algos_length,
+					     gnutls_pk_algorithm
+					     algo_to_check)
 {
 	int i;
 	for (i = 0; i < pk_algos_length; i++) {
@@ -164,20 +121,22 @@ inline
 	return -1;
 }
 
+
+
 /* Locates the most appropriate x509 certificate using the
  * given DN. If indx == -1 then no certificate was found.
  */
 static int _find_x509_cert(const gnutls_certificate_credentials cred,
 			   opaque * _data, size_t _data_size,
-			   gnutls_pk_algorithm * pk_algos, int pk_algos_length,
-			   int *indx)
+			   gnutls_pk_algorithm * pk_algos,
+			   int pk_algos_length, int *indx)
 {
 	uint size;
 	gnutls_datum odn;
 	opaque *data = _data;
 	ssize_t data_size = _data_size;
 	uint i, j;
-	int result;
+	int result, cert_pk;
 
 	*indx = -1;
 
@@ -191,9 +150,7 @@ static int _find_x509_cert(const gnutls_certificate_credentials cred,
 		for (i = 0; i < cred->ncerts; i++) {
 			for (j = 0; j < cred->cert_list_length[i]; j++) {
 				if ((result =
-				     _gnutls_find_dn(&odn,
-						     &cred->cert_list[i]
-						     [j])) < 0) {
+				     _gnutls_cert_get_dn( &cred->cert_list[i][j], &odn)) < 0) {
 					gnutls_assert();
 					return result;
 				}
@@ -205,14 +162,10 @@ static int _find_x509_cert(const gnutls_certificate_credentials cred,
 				 * the *_SIGN algorithm matches
 				 * the cert is our cert!
 				 */
-				if ((memcmp(odn.data,
-					    data, size) == 0) &&
-				    (_gnutls_check_pk_algo_in_list
-				     (pk_algos, pk_algos_length,
-				      cred->cert_list[i][0].
-				      subject_pk_algorithm) == 0)
-				    && (cred->cert_list[i][0].cert_type ==
-					GNUTLS_CRT_X509)) {
+				cert_pk = cred->cert_list[i][0].subject_pk_algorithm;
+
+				if ((memcmp(odn.data, data, size) == 0) &&
+				    (_gnutls_check_pk_algo_in_list(pk_algos, pk_algos_length, cert_pk) == 0)) {
 					*indx = i;
 					break;
 				}
@@ -236,8 +189,8 @@ static int _find_x509_cert(const gnutls_certificate_credentials cred,
 /* Locates the most appropriate openpgp cert
  */
 static int _find_openpgp_cert(const gnutls_certificate_credentials cred,
-			      gnutls_pk_algorithm * pk_algos, int pk_algos_length,
-			      int *indx)
+			      gnutls_pk_algorithm * pk_algos,
+			      int pk_algos_length, int *indx)
 {
 	uint i, j;
 
@@ -266,6 +219,10 @@ static int _find_openpgp_cert(const gnutls_certificate_credentials cred,
 	return 0;
 }
 
+/* Some upper limit to protect alloca();
+ */
+#define MAX_ISSUERS 512
+
 /* Finds the appropriate certificate depending on the cA Distinguished name
  * advertized by the server. If none matches then returns 0 and -1 as index.
  * In case of an error a negative value, is returned.
@@ -276,38 +233,30 @@ static int _find_openpgp_cert(const gnutls_certificate_credentials cred,
 static int _gnutls_find_acceptable_client_cert(gnutls_session session,
 					       opaque * _data,
 					       size_t _data_size, int *ind,
-					       gnutls_pk_algorithm * pk_algos,
+					       gnutls_pk_algorithm *
+					       pk_algos,
 					       int pk_algos_length)
 {
 	int result, size;
 	int indx = -1;
 	uint i, j;
-	int try = 0, *ij_map = NULL;
+	int *ij_map = NULL;
 	const gnutls_certificate_credentials cred;
 	opaque *data = _data;
 	ssize_t data_size = _data_size;
+	gnutls_datum *my_certs = NULL;
+	gnutls_datum *issuers_dn = NULL;
 
 	cred =
-	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE,
-			     NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
-	if (session->internals.client_cert_callback != NULL) {
-		/* if try>=0 then the client wants automatic
-		 * choose of certificate, otherwise (-1), he
-		 * will be prompted to choose one.
-		 */
-		try =
-		    session->internals.client_cert_callback(session,
-								 NULL, 0,
-								 NULL, 0);
-	}
-
-
-	if (try >= 0) {
+	/* If we have no callback.
+	 */
+	if (session->internals.client_cert_callback == NULL) {
 		result = 0;
 
 		if (session->security_parameters.cert_type ==
@@ -328,92 +277,113 @@ static int _gnutls_find_acceptable_client_cert(gnutls_session session,
 			gnutls_assert();
 			return result;
 		}
-	}
-
-
-	/* use the callback 
-	 */
-	if (indx == -1 && session->internals.client_cert_callback != NULL && cred->ncerts > 0) {	/* use a callback to get certificate */
-		gnutls_datum *my_certs = NULL;
-		gnutls_datum *issuers_dn = NULL;
+	} else /* If the callback is set, then use it. */
+	if (session->internals.client_cert_callback != NULL) {
+		/* use a callback to get certificate 
+		 */
 		uint issuers_dn_len = 0;
-		opaque* dataptr = data;
+		opaque *dataptr = data;
 		ssize_t dataptr_size = data_size;
 
 		/* Count the number of the given issuers;
 		 * This is used to allocate the issuers_dn without
 		 * using realloc().
 		 */
-		do {
-			/* This works like DECR_LEN() */
-			DECR_LENGTH_COM( dataptr_size, 2, goto clear);
-			size = _gnutls_read_uint16(dataptr);
 
-			DECR_LENGTH_COM( dataptr_size, size, goto clear);
-
-			dataptr += 2;
-
-			if (size > 0) {
-				issuers_dn_len++;
-				dataptr += size;
-			}
-			
-			if (dataptr_size == 0)
-				break;
-
-		} while (1);
-
-
-		my_certs =
-		    gnutls_alloca(cred->ncerts * sizeof(gnutls_datum));
-		if (my_certs == NULL)
-			goto clear;
-
-		/* put the requested DNs to req_dn, only in case
-		 * of X509 certificates.
-		 */
 		if (gnutls_certificate_type_get(session) == GNUTLS_CRT_X509) {
-			data = _data;
-			data_size = _data_size;
 
-			issuers_dn = gnutls_alloca( issuers_dn_len * sizeof(gnutls_datum));
-			if (issuers_dn == NULL)
-				goto clear;
-
-			for (i=0;i<issuers_dn_len;i++) {
-				/* The checks here for the buffer boundaries
-				 * are not needed since the buffer has been
-				 * parsed above.
+			/* Makes the issuers_dn stuff.
+			 */
+			do {
+				/* This works like DECR_LEN() 
 				 */
-				data_size -= 2;
+				result = GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+				DECR_LENGTH_COM(dataptr_size, 2, goto error);
+				size = _gnutls_read_uint16(dataptr);
 
-				size = _gnutls_read_uint16(data);
+				result = GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+				DECR_LENGTH_COM(dataptr_size, size, goto error);
 
-				data += 2;
+				dataptr += 2;
+		
+				if (size > 0) {
+					issuers_dn_len++;
+					dataptr += size;
+				}
 
-				issuers_dn[i].data = data;
-				issuers_dn[i].size = size;
+				if (dataptr_size == 0)
+					break;
 
-				data += size;
+			} while (issuers_dn_len < MAX_ISSUERS);
 
+			if (cred->ncerts != 0) {
+				my_certs =
+				    gnutls_alloca(cred->ncerts * sizeof(gnutls_datum));
+				if (my_certs == NULL) {
+					result = GNUTLS_E_MEMORY_ERROR;
+					gnutls_assert();
+					goto error;
+				}
 			}
 
-		} else {	/* Other certificate types */
+			/* put the requested DNs to req_dn, only in case
+			 * of X509 certificates.
+			 */
+			if (issuers_dn_len > 0) {
+				data = _data;
+				data_size = _data_size;
+
+				issuers_dn =
+				    gnutls_alloca(issuers_dn_len *
+						  sizeof(gnutls_datum));
+				if (issuers_dn == NULL) {
+					result = GNUTLS_E_MEMORY_ERROR;
+					gnutls_assert();
+					goto error;
+				}
+
+				for (i = 0; i < issuers_dn_len; i++) {
+					/* The checks here for the buffer boundaries
+					 * are not needed since the buffer has been
+					 * parsed above.
+					 */
+					data_size -= 2;
+
+					size = _gnutls_read_uint16(data);
+
+					data += 2;
+
+					issuers_dn[i].data = data;
+					issuers_dn[i].size = size;
+	
+					data += size;
+
+				}
+			}
+
+		} else { /* Other certificate types */
 			issuers_dn_len = 0;
 			issuers_dn = NULL;
 		}
 
-		/* maps j -> i */
-		ij_map = gnutls_alloca(sizeof(int) * cred->ncerts);
-		if (ij_map==NULL) {
-			gnutls_assert();
-			goto clear;
+		/* If not certificates are present.
+		 */
+		/* maps j -> i 
+		 */
+
+		if (cred->ncerts != 0) {
+			ij_map = gnutls_alloca(sizeof(int) * cred->ncerts);
+			if (ij_map == NULL) {
+				result = GNUTLS_E_MEMORY_ERROR;
+				gnutls_assert();
+				goto error;
+			}
 		}
 
 		/* put our certificate's issuer and dn into cdn, idn
 		 * Note that the certificates we provide to the callback
 		 * are not all the certificates we have. Only the certificates
-		 * that are requested by the server (CA matches - and sign
+		 * that are requested by the server (certificate type - and sign
 		 * algorithm matches), are provided.
 		 */
 		for (j = i = 0; i < cred->ncerts; i++) {
@@ -435,41 +405,50 @@ static int _gnutls_find_acceptable_client_cert(gnutls_session session,
 
 		indx =
 		    session->internals.client_cert_callback(session,
-								 my_certs,
-								 j,
-								 issuers_dn,
-								 issuers_dn_len);
+							    my_certs,
+							    j,
+							    issuers_dn,
+							    issuers_dn_len);
 
 		/* the indx returned by the user is relative
 		 * to the certificates we provided him.
 		 * This will make it relative to the certificates
 		 * we've got.
 		 */
-		if (indx != -1)
+		if (indx != -1 && cred->ncerts != 0)
 			indx = ij_map[indx];
+		else
+			indx = -1;
 
-	      clear:
-		gnutls_afree(my_certs);
-		gnutls_afree(ij_map);
-		gnutls_afree(issuers_dn);
+		if (my_certs) gnutls_afree(my_certs);
+		if (ij_map) gnutls_afree(ij_map);
+		if (issuers_dn) gnutls_afree(issuers_dn);
 	}
 
 	*ind = indx;
 	return 0;
+
+	error:
+		if (my_certs != NULL) { gnutls_afree(my_certs); }
+		if (ij_map != NULL) { gnutls_afree(ij_map); }
+		if (issuers_dn != NULL) { gnutls_afree(issuers_dn); }
+		return result;
+
 }
 
 /* Generate client certificate
  */
 
-int _gnutls_gen_x509_certificate(gnutls_session session, opaque ** data)
+int _gnutls_gen_x509_crt(gnutls_session session, opaque ** data)
 {
 	int ret, i;
 	opaque *pdata;
 	gnutls_cert *apr_cert_list;
-	gnutls_private_key *apr_pkey;
+	gnutls_privkey *apr_pkey;
 	int apr_cert_list_length;
 
-	/* find the appropriate certificate */
+	/* find the appropriate certificate 
+	 */
 	if ((ret =
 	     _gnutls_find_apr_cert(session, &apr_cert_list,
 				   &apr_cert_list_length,
@@ -513,13 +492,12 @@ int _gnutls_gen_x509_certificate(gnutls_session session, opaque ** data)
 
 enum PGPKeyDescriptorType { PGP_KEY_FINGERPRINT, PGP_KEY };
 
-int _gnutls_gen_openpgp_certificate(gnutls_session session,
-					   opaque ** data)
+int _gnutls_gen_openpgp_certificate(gnutls_session session, opaque ** data)
 {
 	int ret;
 	opaque *pdata;
 	gnutls_cert *apr_cert_list;
-	gnutls_private_key *apr_pkey;
+	gnutls_privkey* apr_pkey;
 	int apr_cert_list_length;
 
 	/* find the appropriate certificate */
@@ -564,12 +542,13 @@ OPENPGP_KEY_REQUEST _E_gnutls_openpgp_request_key = NULL;
 extern OPENPGP_CERT2GNUTLS_CERT _E_gnutls_openpgp_cert2gnutls_cert;
 
 int _gnutls_gen_openpgp_certificate_fpr(gnutls_session session,
-					       opaque ** data)
+					opaque ** data)
 {
-	int ret, fpr_size, packet_size;
+	int ret, packet_size;
+	size_t fpr_size;
 	opaque *pdata;
 	gnutls_cert *apr_cert_list;
-	gnutls_private_key *apr_pkey;
+	gnutls_privkey* apr_pkey;
 	int apr_cert_list_length;
 
 	/* find the appropriate certificate */
@@ -609,12 +588,14 @@ int _gnutls_gen_openpgp_certificate_fpr(gnutls_session session,
 
 	fpr_size = 20;
 
-	if (_E_gnutls_openpgp_fingerprint==NULL) {
+	if (_E_gnutls_openpgp_fingerprint == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INIT_LIBEXTRA;
 	}
 
-	if ( (ret=_E_gnutls_openpgp_fingerprint( &apr_cert_list[0].raw, pdata, &fpr_size)) < 0) {
+	if ((ret =
+	     _E_gnutls_openpgp_fingerprint(&apr_cert_list[0].raw, pdata,
+					   &fpr_size)) < 0) {
 		gnutls_assert();
 		return ret;
 	}
@@ -624,21 +605,21 @@ int _gnutls_gen_openpgp_certificate_fpr(gnutls_session session,
 
 
 
-int _gnutls_gen_cert_client_certificate(gnutls_session session, opaque ** data)
+int _gnutls_gen_cert_client_certificate(gnutls_session session,
+					opaque ** data)
 {
 	switch (session->security_parameters.cert_type) {
 	case GNUTLS_CRT_OPENPGP:
 		if (_gnutls_openpgp_send_fingerprint(session) == 0)
 			return
-			    _gnutls_gen_openpgp_certificate(session,
-								   data);
+			    _gnutls_gen_openpgp_certificate(session, data);
 		else
 			return
 			    _gnutls_gen_openpgp_certificate_fpr
 			    (session, data);
 
 	case GNUTLS_CRT_X509:
-		return _gnutls_gen_x509_certificate(session, data);
+		return _gnutls_gen_x509_crt(session, data);
 
 	default:
 		gnutls_assert();
@@ -646,13 +627,14 @@ int _gnutls_gen_cert_client_certificate(gnutls_session session, opaque ** data)
 	}
 }
 
-int _gnutls_gen_cert_server_certificate(gnutls_session session, opaque ** data)
+int _gnutls_gen_cert_server_certificate(gnutls_session session,
+					opaque ** data)
 {
 	switch (session->security_parameters.cert_type) {
 	case GNUTLS_CRT_OPENPGP:
 		return _gnutls_gen_openpgp_certificate(session, data);
 	case GNUTLS_CRT_X509:
-		return _gnutls_gen_x509_certificate(session, data);
+		return _gnutls_gen_x509_crt(session, data);
 	default:
 		gnutls_assert();
 		return GNUTLS_E_INTERNAL_ERROR;
@@ -662,9 +644,9 @@ int _gnutls_gen_cert_server_certificate(gnutls_session session, opaque ** data)
 /* Process server certificate
  */
 
-#define CLEAR_CERTS for(x=0;x<peer_certificate_list_size;x++) _gnutls_free_cert(peer_certificate_list[x])
-int _gnutls_proc_x509_server_certificate(gnutls_session session, opaque * data,
-					 size_t data_size)
+#define CLEAR_CERTS for(x=0;x<peer_certificate_list_size;x++) _gnutls_free_cert(&peer_certificate_list[x])
+int _gnutls_proc_x509_server_certificate(gnutls_session session,
+					 opaque * data, size_t data_size)
 {
 	int size, len, ret;
 	opaque *p = data;
@@ -677,11 +659,10 @@ int _gnutls_proc_x509_server_certificate(gnutls_session session, opaque * data,
 	gnutls_datum tmp;
 
 	cred =
-	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE,
-			     NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
 
@@ -712,7 +693,7 @@ int _gnutls_proc_x509_server_certificate(gnutls_session session, opaque * data,
 	}
 
 	i = dsize;
-	while(i > 0) {
+	while (i > 0) {
 		DECR_LEN(dsize, 3);
 		len = _gnutls_read_uint24(p);
 		p += 3;
@@ -732,15 +713,15 @@ int _gnutls_proc_x509_server_certificate(gnutls_session session, opaque * data,
 	 */
 
 	peer_certificate_list =
-	    gnutls_alloca( sizeof(gnutls_cert) *
+	    gnutls_malloc(sizeof(gnutls_cert) *
 			  (peer_certificate_list_size));
 
 	if (peer_certificate_list == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
-	memset( peer_certificate_list, 0, sizeof(gnutls_cert)*
-					peer_certificate_list_size);
+	memset(peer_certificate_list, 0, sizeof(gnutls_cert) *
+	       peer_certificate_list_size);
 
 	p = data + 3;
 
@@ -749,7 +730,7 @@ int _gnutls_proc_x509_server_certificate(gnutls_session session, opaque * data,
 	 * been parsed before.
 	 */
 
-	for (j=0;j<peer_certificate_list_size;j++) {
+	for (j = 0; j < peer_certificate_list_size; j++) {
 		len = _gnutls_read_uint24(p);
 		p += 3;
 
@@ -758,11 +739,10 @@ int _gnutls_proc_x509_server_certificate(gnutls_session session, opaque * data,
 
 		if ((ret =
 		     _gnutls_x509_cert2gnutls_cert(&peer_certificate_list
-						   [j], tmp, CERT_ONLY_EXTENSIONS)) < 0) {
+						   [j], &tmp,
+						   CERT_ONLY_EXTENSIONS)) < 0) {
 			gnutls_assert();
-			CLEAR_CERTS;
-			gnutls_afree(peer_certificate_list);
-			return ret;
+			goto cleanup;
 		}
 
 		p += len;
@@ -770,35 +750,33 @@ int _gnutls_proc_x509_server_certificate(gnutls_session session, opaque * data,
 
 
 	if ((ret =
-	     _gnutls_copy_certificate_auth_info(info,
+	     _gnutls_copy_certificate_auth_info(info, 
 						peer_certificate_list,
-						peer_certificate_list_size))
-	    < 0) {
+						peer_certificate_list_size)) < 0) {
 		gnutls_assert();
-		CLEAR_CERTS;
-		gnutls_afree(peer_certificate_list);
-		return ret;
+		goto cleanup;
 	}
 
 	if ((ret =
-	     _gnutls_check_x509_key_usage(&peer_certificate_list[0],
-					  gnutls_kx_get(session)))
-	    < 0) {
+	     _gnutls_check_key_usage(&peer_certificate_list[0],
+					  gnutls_kx_get(session))) < 0) {
 		gnutls_assert();
-		CLEAR_CERTS;
-		gnutls_afree(peer_certificate_list);
-		return ret;
+		goto cleanup;
 	}
 
+	ret = 0;
+
+cleanup:
 	CLEAR_CERTS;
-	gnutls_afree(peer_certificate_list);
+	gnutls_free(peer_certificate_list);
+	return ret;
 
-	return 0;
 }
 
-#define CLEAR_CERTS for(x=0;x<peer_certificate_list_size;x++) _gnutls_free_cert(peer_certificate_list[x])
+#define CLEAR_CERTS for(x=0;x<peer_certificate_list_size;x++) _gnutls_free_cert(&peer_certificate_list[x])
 int _gnutls_proc_openpgp_server_certificate(gnutls_session session,
-					    opaque * data, size_t data_size)
+					    opaque * data,
+					    size_t data_size)
 {
 	int size, ret, len;
 	opaque *p = data;
@@ -806,16 +784,15 @@ int _gnutls_proc_openpgp_server_certificate(gnutls_session session,
 	const gnutls_certificate_credentials cred;
 	ssize_t dsize = data_size;
 	int i, x;
-	gnutls_cert *peer_certificate_list;
+	gnutls_cert *peer_certificate_list = NULL;
 	int peer_certificate_list_size = 0;
 	gnutls_datum tmp, akey = { NULL, 0 };
 
 	cred =
-	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE,
-			     NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
 	if ((ret =
@@ -846,35 +823,37 @@ int _gnutls_proc_openpgp_server_certificate(gnutls_session session,
 
 	/* Read PGPKeyDescriptor */
 	DECR_LEN(dsize, 1);
-	if (*p == PGP_KEY_FINGERPRINT) { /* the fingerprint */
+	if (*p == PGP_KEY_FINGERPRINT) {	/* the fingerprint */
 		p++;
-		
+
 		DECR_LEN(dsize, 1);
-		len = (uint8) *p;
-                p++;
-		
+		len = (uint8) * p;
+		p++;
+
 		if (len != 20) {
 			gnutls_assert();
-			return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+			return GNUTLS_E_OPENPGP_FINGERPRINT_UNSUPPORTED;
 		}
-		
-		DECR_LEN( dsize, 20);
+
+		DECR_LEN(dsize, 20);
 
 		/* request the actual key from our database, or
 		 * a key server or anything.
 		 */
-		if (_E_gnutls_openpgp_request_key==NULL) {
+		if (_E_gnutls_openpgp_request_key == NULL) {
 			gnutls_assert();
 			return GNUTLS_E_INIT_LIBEXTRA;
 		}
-		if ( (ret=_E_gnutls_openpgp_request_key( session, &akey, cred, p, 20)) < 0) {
+		if ((ret =
+		     _E_gnutls_openpgp_request_key(session, &akey, cred, p,
+						   20)) < 0) {
 			gnutls_assert();
 			return ret;
 		}
 		tmp = akey;
 		peer_certificate_list_size++;
 
-	} else if (*p == PGP_KEY) { /* the whole key */
+	} else if (*p == PGP_KEY) {	/* the whole key */
 
 		p++;
 
@@ -897,7 +876,7 @@ int _gnutls_proc_openpgp_server_certificate(gnutls_session session,
 
 	} else {
 		gnutls_assert();
-		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+		return GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE;
 	}
 
 	/* ok we now have the peer's key in tmp datum
@@ -905,72 +884,66 @@ int _gnutls_proc_openpgp_server_certificate(gnutls_session session,
 
 	if (peer_certificate_list_size == 0) {
 		gnutls_assert();
-		_gnutls_free_datum( &akey);
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
 	peer_certificate_list =
-	    gnutls_alloca( sizeof(gnutls_cert) *
+	    gnutls_alloca(sizeof(gnutls_cert) *
 			  (peer_certificate_list_size));
-		if (peer_certificate_list == NULL) {
+	if (peer_certificate_list == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		ret = GNUTLS_E_MEMORY_ERROR;
+		goto cleanup;
 	}
-	memset( peer_certificate_list, 0, sizeof(gnutls_cert)*
-			peer_certificate_list_size);
+	memset(peer_certificate_list, 0, sizeof(gnutls_cert) *
+	       peer_certificate_list_size);
 
-	if (_E_gnutls_openpgp_cert2gnutls_cert==NULL) {
+	if (_E_gnutls_openpgp_cert2gnutls_cert == NULL) {
 		gnutls_assert();
-		_gnutls_free_datum( &akey);
-		CLEAR_CERTS;
-		gnutls_afree(peer_certificate_list);
-		return GNUTLS_E_INIT_LIBEXTRA;
+		ret = GNUTLS_E_INIT_LIBEXTRA;
+		goto cleanup;
 	}
-	
+
 	if ((ret =
 	     _E_gnutls_openpgp_cert2gnutls_cert(&peer_certificate_list[0],
-					      tmp)) < 0) {
+						&tmp)) < 0) {
 		gnutls_assert();
-		_gnutls_free_datum( &akey);
-		CLEAR_CERTS;
-		gnutls_afree(peer_certificate_list);
-		return ret;
+		goto cleanup;
 	}
-	_gnutls_free_datum( &akey);
 
 	if ((ret =
 	     _gnutls_copy_certificate_auth_info(info,
-						peer_certificate_list,
-						peer_certificate_list_size))
-	    < 0) {
+						peer_certificate_list, 
+						peer_certificate_list_size)) < 0) {
 		gnutls_assert();
-		CLEAR_CERTS;
-		gnutls_afree(peer_certificate_list);
-		return ret;
+		goto cleanup;
 	}
 
 	if ((ret =
-	     _gnutls_check_x509_key_usage(&peer_certificate_list[0],
-					  gnutls_kx_get(session)))
-	    < 0) {
+	     _gnutls_check_key_usage(&peer_certificate_list[0],
+					  gnutls_kx_get(session))) < 0) {
 		gnutls_assert();
-		CLEAR_CERTS;
-		gnutls_afree(peer_certificate_list);
-		return ret;
+		goto cleanup;
 	}
 
+	ret = 0;
+	
+cleanup:
+
+	_gnutls_free_datum(&akey);
 	CLEAR_CERTS;
 	gnutls_afree(peer_certificate_list);
-
-	return 0;
+	return ret;
+	
 }
 
-int _gnutls_proc_cert_server_certificate(gnutls_session session, opaque * data,
-					 size_t data_size)
+int _gnutls_proc_cert_server_certificate(gnutls_session session,
+					 opaque * data, size_t data_size)
 {
 	switch (session->security_parameters.cert_type) {
 	case GNUTLS_CRT_OPENPGP:
-		return _gnutls_proc_openpgp_server_certificate(session, data,
+		return _gnutls_proc_openpgp_server_certificate(session,
+							       data,
 							       data_size);
 	case GNUTLS_CRT_X509:
 		return _gnutls_proc_x509_server_certificate(session, data,
@@ -1015,11 +988,10 @@ int _gnutls_proc_cert_cert_req(gnutls_session session, opaque * data,
 	int pk_algos_length;
 
 	cred =
-	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE,
-			     NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
 	if ((ret =
@@ -1078,7 +1050,7 @@ int _gnutls_proc_cert_cert_req(gnutls_session session, opaque * data,
 	 */
 	session->internals.selected_cert_index = ind;
 
-	/* We should reply with a certificate message,
+	/* We should reply with a certificate message, 
 	 * even if we have no certificate to send.
 	 */
 	session->key->certificate_requested = 1;
@@ -1086,11 +1058,12 @@ int _gnutls_proc_cert_cert_req(gnutls_session session, opaque * data,
 	return 0;
 }
 
-int _gnutls_gen_cert_client_cert_vrfy(gnutls_session session, opaque ** data)
+int _gnutls_gen_cert_client_cert_vrfy(gnutls_session session,
+				      opaque ** data)
 {
 	int ret;
 	gnutls_cert *apr_cert_list;
-	gnutls_private_key *apr_pkey;
+	gnutls_privkey* apr_pkey;
 	int apr_cert_list_length, size;
 	gnutls_datum signature;
 
@@ -1107,7 +1080,7 @@ int _gnutls_gen_cert_client_cert_vrfy(gnutls_session session, opaque ** data)
 
 	if (apr_pkey != NULL) {
 		if ((ret =
-		     _gnutls_generate_sig_from_hdata(session,
+		     _gnutls_tls_sign_hdata(session,
 						     &apr_cert_list[0],
 						     apr_pkey,
 						     &signature)) < 0) {
@@ -1134,8 +1107,8 @@ int _gnutls_gen_cert_client_cert_vrfy(gnutls_session session, opaque ** data)
 	return size + 2;
 }
 
-int _gnutls_proc_cert_client_cert_vrfy(gnutls_session session, opaque * data,
-				       size_t data_size)
+int _gnutls_proc_cert_client_cert_vrfy(gnutls_session session,
+				       opaque * data, size_t data_size)
 {
 	int size, ret;
 	ssize_t dsize = data_size;
@@ -1163,19 +1136,20 @@ int _gnutls_proc_cert_client_cert_vrfy(gnutls_session session, opaque * data,
 	case GNUTLS_CRT_X509:
 		ret =
 		    _gnutls_x509_cert2gnutls_cert(&peer_cert,
-						  info->
-						  raw_certificate_list[0], CERT_NO_COPY);
+						  &info->
+						  raw_certificate_list[0],
+						  CERT_NO_COPY);
 		break;
 	case GNUTLS_CRT_OPENPGP:
-		if (_E_gnutls_openpgp_cert2gnutls_cert==NULL) {
+		if (_E_gnutls_openpgp_cert2gnutls_cert == NULL) {
 			gnutls_assert();
 			return GNUTLS_E_INIT_LIBEXTRA;
 		}
 		ret =
 		    _E_gnutls_openpgp_cert2gnutls_cert(&peer_cert,
-						     info->
-						     raw_certificate_list
-						     [0]);
+						       &info->
+						       raw_certificate_list
+						       [0]);
 		break;
 	default:
 		gnutls_assert();
@@ -1190,16 +1164,17 @@ int _gnutls_proc_cert_client_cert_vrfy(gnutls_session session, opaque * data,
 	if ((ret =
 	     _gnutls_verify_sig_hdata(session, &peer_cert, &sig)) < 0) {
 		gnutls_assert();
-		_gnutls_free_cert(peer_cert);
+		_gnutls_free_cert(&peer_cert);
 		return ret;
 	}
-	_gnutls_free_cert(peer_cert);
+	_gnutls_free_cert(&peer_cert);
 
 	return 0;
 }
 
 #define CERTTYPE_SIZE 3
-int _gnutls_gen_cert_server_cert_req(gnutls_session session, opaque ** data)
+int _gnutls_gen_cert_server_cert_req(gnutls_session session,
+				     opaque ** data)
 {
 	const gnutls_certificate_credentials cred;
 	int size;
@@ -1211,17 +1186,17 @@ int _gnutls_gen_cert_server_cert_req(gnutls_session session, opaque ** data)
 	 */
 
 	cred =
-	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE,
-			     NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
 	size = CERTTYPE_SIZE + 2;	/* 2 for gnutls_certificate_type + 2 for size of rdn_seq 
 					 */
 
-	if (session->security_parameters.cert_type == GNUTLS_CRT_X509)
+	if (session->security_parameters.cert_type == GNUTLS_CRT_X509 &&
+		session->internals.ignore_rdn_sequence == 0)
 		size += cred->x509_rdn_sequence.size;
 
 	(*data) = gnutls_malloc(size);
@@ -1238,35 +1213,39 @@ int _gnutls_gen_cert_server_cert_req(gnutls_session session, opaque ** data)
 	pdata[2] = DSA_SIGN;	/* only these for now */
 	pdata += CERTTYPE_SIZE;
 
-	if (session->security_parameters.cert_type == GNUTLS_CRT_X509) {
+	if (session->security_parameters.cert_type == GNUTLS_CRT_X509 &&
+		session->internals.ignore_rdn_sequence == 0) {
 		_gnutls_write_datum16(pdata, cred->x509_rdn_sequence);
-		pdata += cred->x509_rdn_sequence.size + 2;
+		/* pdata += cred->x509_rdn_sequence.size + 2; */
 	}
 
 	return size;
 }
 
 
-/* This function will return the appropriate certificate to use. The return
- * value depends on the side (client or server).
+/* This function will return the appropriate certificate to use. 
+ * Fills in the apr_cert_list, apr_cert_list_length and apr_pkey.
+ * The return value is a negative value on error.
+ *
+ * It is normal to return 0 with no certificates in client side.
+ *
  */
-int _gnutls_find_apr_cert(gnutls_session session, gnutls_cert ** apr_cert_list,
+int _gnutls_find_apr_cert(gnutls_session session,
+			  gnutls_cert ** apr_cert_list,
 			  int *apr_cert_list_length,
-			  gnutls_private_key ** apr_pkey)
+			  gnutls_privkey ** apr_pkey)
 {
 	const gnutls_certificate_credentials cred;
 	int ind;
 
-	cred =
-	    _gnutls_get_kx_cred(session, GNUTLS_CRD_CERTIFICATE,
-				NULL);
+	cred = _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 
 	if (cred == NULL) {
 		gnutls_assert();
 		*apr_cert_list = NULL;
 		*apr_pkey = NULL;
 		*apr_cert_list_length = 0;
-		return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
 	if (session->security_parameters.entity == GNUTLS_SERVER) {
@@ -1276,7 +1255,7 @@ int _gnutls_find_apr_cert(gnutls_session session, gnutls_cert ** apr_cert_list,
 			*apr_cert_list_length = 0;
 			*apr_pkey = NULL;
 			gnutls_assert();	/* this is not allowed */
-			return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+			return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 		} else {
 			/* find_cert_list_index() has been called before.
 			 */
@@ -1287,7 +1266,7 @@ int _gnutls_find_apr_cert(gnutls_session session, gnutls_cert ** apr_cert_list,
 				*apr_cert_list_length = 0;
 				*apr_pkey = NULL;
 				gnutls_assert();
-				return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+				return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 			} else {
 				*apr_cert_list = cred->cert_list[ind];
 				*apr_cert_list_length =
@@ -1330,20 +1309,19 @@ int _gnutls_find_apr_cert(gnutls_session session, gnutls_cert ** apr_cert_list,
  * (frontend to _gnutls_server_find_cert_index())
  */
 const gnutls_cert *_gnutls_server_find_cert(gnutls_session session,
-					    gnutls_pk_algorithm requested_algo)
+					    gnutls_pk_algorithm
+					    requested_algo)
 {
 	int i;
 	const gnutls_certificate_credentials x509_cred;
 
 	x509_cred =
-	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE,
-			     NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 
 	if (x509_cred == NULL)
 		return NULL;
 
-	i = _gnutls_server_find_cert_list_index(session,
-						requested_algo);
+	i = _gnutls_server_find_cert_list_index(session, requested_algo);
 
 	if (i < 0)
 		return NULL;
@@ -1358,27 +1336,26 @@ const gnutls_cert *_gnutls_server_find_cert(gnutls_session session,
  * or -1 for any).
  */
 static int _gnutls_server_find_cert_list_index(gnutls_session session,
-					gnutls_pk_algorithm requested_algo)
+					       gnutls_pk_algorithm
+					       requested_algo)
 {
 	uint i, j;
-	int index = -1;
+	int index;
 	const gnutls_certificate_credentials cred;
 	int my_certs_length;
-	int *ij_map = NULL;
 
 	cred =
-	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE,
-			     NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_CERTIFICATE, NULL);
 	if (cred == NULL) {
 		gnutls_assert();
-		return GNUTLS_E_INSUFICIENT_CREDENTIALS;
+		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 
 	index = -1;		/* default is use no certificate */
 
 	for (i = 0; i < cred->ncerts; i++) {
 		/* find one compatible certificate */
-		if (requested_algo == (gnutls_pk_algorithm)-1 ||
+		if (requested_algo == (gnutls_pk_algorithm) - 1 ||
 		    requested_algo ==
 		    cred->cert_list[i][0].subject_pk_algorithm) {
 			/* if cert type matches */
@@ -1391,28 +1368,33 @@ static int _gnutls_server_find_cert_list_index(gnutls_session session,
 
 	}
 
-	if (session->internals.server_cert_callback != NULL && cred->ncerts > 0) {	/* use the callback to get certificate */
-		gnutls_datum *my_certs = NULL;
+	if (session->internals.server_cert_callback != NULL && cred->ncerts > 0) {	
+		/* use the callback to get certificate 
+		 */
+		gnutls_datum *my_certs;
+		int *ij_map;
 
-		my_certs =
-		    gnutls_malloc(cred->ncerts * sizeof(gnutls_datum));
-		if (my_certs == NULL)
-			goto clear;
 		my_certs_length = cred->ncerts;
+		my_certs =
+		    gnutls_malloc(my_certs_length * sizeof(gnutls_datum));
+		if (my_certs == NULL)
+			goto out;
 
 		/* put our certificate's issuer and dn into cdn, idn
 		 */
-		ij_map = gnutls_malloc(sizeof(int) * cred->ncerts);
+		ij_map = gnutls_malloc(my_certs_length * sizeof(int));
+		if (ij_map == NULL) goto cleanup_certs;
 
 		j = 0;
 		for (i = 0; i < cred->ncerts; i++) {
 			/* Add compatible certificates */
-			if (requested_algo == (gnutls_pk_algorithm)-1 ||
+			if (requested_algo == (gnutls_pk_algorithm) - 1 ||
 			    requested_algo ==
 			    cred->cert_list[i][0].subject_pk_algorithm) {
 
 				/* if cert type matches */
-				if (session->security_parameters.cert_type ==
+				if (session->security_parameters.
+				    cert_type ==
 				    cred->cert_list[i][0].cert_type) {
 
 					ij_map[j] = i;
@@ -1425,17 +1407,19 @@ static int _gnutls_server_find_cert_list_index(gnutls_session session,
 
 		index =
 		    session->internals.server_cert_callback(session,
-								 my_certs,
-								 my_certs_length);
+							    my_certs,
+							    my_certs_length);
 
 		if (index != -1)
 			index = ij_map[index];
 
-	      clear:
-		gnutls_free(my_certs);
 		gnutls_free(ij_map);
+		cleanup_certs:
+		gnutls_free(my_certs);
+
 	}
 
+	out:
 	/* store the index for future use, in the handshake.
 	 * (This will allow not calling this callback again.)
 	 */
