@@ -26,131 +26,88 @@
 
 #ifdef DEBUG
 
-void _gnutls_dump_mpi(char* prefix, GNUTLS_MPI a)
-{
-	char buf[1024];
-	size_t n = sizeof buf;
-	
-	if (gcry_mpi_print(GCRYMPI_FMT_HEX, buf, &n, a))
-		strcpy(buf, "[can't print value]"); /* Flawfinder: ignore */
-	_gnutls_log( "GNUTLS_MPI: length: %d\n\t%s%s\n", (n-1)/2, prefix, buf);
-}
-
-
-char *_gnutls_bin2hex(const unsigned char *old, const size_t oldlen)
-{
-	unsigned char *new = NULL;
-	int i, j;
-
-	new = calloc(1, oldlen * 2 * sizeof(char) + 1);
-	if (!new)
-		return (new);
-
-	for (i = j = 0; i < oldlen; j+=2) {
-		sprintf(&new[j], "%.2x", old[i]); /* Flawfinder: ignore */
-		i++;
-	}
-	new[j] = '\0';
-
-	return (new);
-}
 
 void _gnutls_print_state(gnutls_session session)
 {
 
-	_gnutls_log( "GNUTLS State:\n");
-	_gnutls_log( "Connection End: %d\n",
+	_gnutls_debug_log( "GNUTLS State:\n");
+	_gnutls_debug_log( "Connection End: %d\n",
 		session->security_parameters.entity);
-	_gnutls_log( "Cipher Algorithm: %d\n",
+	_gnutls_debug_log( "Cipher Algorithm: %d\n",
 		session->security_parameters.read_bulk_cipher_algorithm);
-	_gnutls_log( "MAC algorithm: %d\n",
+	_gnutls_debug_log( "MAC algorithm: %d\n",
 		session->security_parameters.read_mac_algorithm);
-	_gnutls_log( "Compression Algorithm: %d\n",
+	_gnutls_debug_log( "Compression Algorithm: %d\n",
 		session->security_parameters.read_compression_algorithm);
-	_gnutls_log( "\n");
+	_gnutls_debug_log( "\n");
 
 }
 
+#endif
 
-const char* _gnutls_packet2str( int packet) {
-static char str[512];
-
+const char* _gnutls_packet2str( ContentType packet) 
+{
 	switch(packet) {
 		case GNUTLS_CHANGE_CIPHER_SPEC:
-			strcpy(str, "Change Cipher Spec");
-			break;		
+			return "Change Cipher Spec";
 		case GNUTLS_ALERT:
-			strcpy(str, "Alert");
-			break;		
+			return "Alert";
 		case GNUTLS_HANDSHAKE:
-			strcpy(str, "Handshake");
-			break;
+			return "Handshake";
 		case GNUTLS_APPLICATION_DATA:
-			strcpy(str, "Application Data");
-			break;
+			return "Application Data";
 
 		default:
-			strcpy(str, "Unknown Packet");
-			
+			return "Unknown Packet";
 	}	
-	return str;	
-	
 }
 
-const char* _gnutls_handshake2str( int handshake) {
-static char str[512];
+const char* _gnutls_handshake2str( HandshakeType handshake) 
+{
 
 	switch(handshake) {
 		case GNUTLS_HELLO_REQUEST:
-			strcpy(str, "HELLO REQUEST");
-			break;		
+			return "HELLO REQUEST";
+			break;
 		case GNUTLS_CLIENT_HELLO:
-			strcpy(str, "CLIENT HELLO");
+			return "CLIENT HELLO";
 			break;		
 		case GNUTLS_SERVER_HELLO:
-			strcpy(str, "SERVER HELLO");
+			return "SERVER HELLO";
 			break;
 		case GNUTLS_CERTIFICATE_PKT:
-			strcpy(str, "CERTIFICATE");
+			return "CERTIFICATE";
 			break;
 		case GNUTLS_SERVER_KEY_EXCHANGE:
-			strcpy(str, "SERVER KEY EXCHANGE");
+			return "SERVER KEY EXCHANGE";
 			break;
 		case GNUTLS_CERTIFICATE_REQUEST:
-			strcpy(str, "CERTIFICATE REQUEST");
+			return "CERTIFICATE REQUEST";
 			break;
 		case GNUTLS_SERVER_HELLO_DONE:
-			strcpy(str, "SERVER HELLO DONE");
+			return "SERVER HELLO DONE";
 			break;
 		case GNUTLS_CERTIFICATE_VERIFY:
-			strcpy(str, "CERTIFICATE VERIFY");
+			return "CERTIFICATE VERIFY";
 			break;
 		case GNUTLS_CLIENT_KEY_EXCHANGE:
-			strcpy(str, "CLIENT KEY EXCHANGE");
+			return "CLIENT KEY EXCHANGE";
 			break;
 		case GNUTLS_FINISHED:
-			strcpy(str, "FINISHED");
+			return "FINISHED";
 			break;
 		default:
-			strcpy(str, "Unknown Handshake");
+			return "Unknown Handshake packet";
 			
 	}	
-	return str;	
+}
+
+void _gnutls_dump_mpi(const char* prefix, GNUTLS_MPI a)
+{
+	opaque buf[1024];
+	size_t n = sizeof buf;
 	
+	if (gcry_mpi_print(GCRYMPI_FMT_HEX, buf, n, &n, a))
+		strcpy(buf, "[can't print value]"); /* Flawfinder: ignore */
+	_gnutls_hard_log( "MPI: length: %d\n\t%s%s\n", (n-1)/2, prefix, buf);
 }
-#endif
-
-#ifdef DEBUG
-int gnutls_x509_extract_certificate_dn( const gnutls_datum*, gnutls_x509_dn*);
-
-const char* GET_CN( gnutls_datum cert) {
-static gnutls_x509_dn dn;
-
-	if (gnutls_x509_extract_certificate_dn( &cert, &dn) >= 0)
-		return &dn.common_name[0];
-	else 
-		return NULL;
-
-}
-
-#endif
