@@ -1,7 +1,7 @@
 /*
- *      Copyright (C) 2000,2001,2002 Nikos Mavroyanopoulos
+ *  Copyright (C) 2000,2001,2002,2003 Nikos Mavroyanopoulos
  *
- * This file is part of GNUTLS.
+ *  This file is part of GNUTLS.
  *
  *  The GNUTLS library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public   
@@ -26,14 +26,14 @@
 #include <defines.h>
 
 /*
+ * They are not needed any more. You can simply enable
+ * the gnutls_log callback to get error descriptions.
+
 #define IO_DEBUG 3 // define this to check non blocking behaviour
 #define BUFFERS_DEBUG
-#define HARD_DEBUG
 #define WRITE_DEBUG
 #define READ_DEBUG
 #define HANDSHAKE_DEBUG // Prints some information on handshake 
-#define X509_DEBUG
-#define RECORD_DEBUG
 #define COMPRESSION_DEBUG
 #define DEBUG
 */
@@ -43,8 +43,6 @@
  */
 typedef void * gnutls_transport_ptr;
 
-#define MIN_BITS 767
-
 #define MAX32 4294967295
 #define MAX24 16777215
 #define MAX16 65535
@@ -62,8 +60,7 @@ typedef void * gnutls_transport_ptr;
  */
 #define MAX_HASH_SIZE 20
 
-#define MAX_X509_CERT_SIZE 10*1024
-#define MAX_LOG_SIZE 1024 /* maximum number of log message */
+#define MAX_LOG_SIZE 1024 /* maximum size of log message */
 #define MAX_SRP_USERNAME 128
 #define MAX_SERVER_NAME_SIZE 128
 
@@ -98,9 +95,9 @@ typedef void * gnutls_transport_ptr;
 #include <gnutls_mem.h>
 #include <gnutls_ui.h>
 
-#define DECR_LEN(len, x) len-=x; if (len<0) {gnutls_assert(); return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;}
-#define DECR_LENGTH_RET(len, x, RET) len-=x; if (len<0) {gnutls_assert(); return RET;}
-#define DECR_LENGTH_COM(len, x, COM) len-=x; if (len<0) {gnutls_assert(); COM;}
+#define DECR_LEN(len, x) do { len-=x; if (len<0) {gnutls_assert(); return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;} } while (0)
+#define DECR_LENGTH_RET(len, x, RET) do { len-=x; if (len<0) {gnutls_assert(); return RET;} } while (0)
+#define DECR_LENGTH_COM(len, x, COM) do { len-=x; if (len<0) {gnutls_assert(); COM;} } while (0)
 
 typedef unsigned char opaque;
 typedef struct { opaque pint[3]; } uint24;
@@ -110,10 +107,18 @@ typedef struct { opaque pint[3]; } uint24;
 typedef enum ChangeCipherSpecType { GNUTLS_TYPE_CHANGE_CIPHER_SPEC=1 } ChangeCipherSpecType;
 
 typedef enum gnutls_certificate_status { 
-	GNUTLS_CERT_NOT_TRUSTED=2, 
-	GNUTLS_CERT_INVALID=4, 
-	GNUTLS_CERT_CORRUPTED=16,
-	GNUTLS_CERT_REVOKED=32
+	GNUTLS_CERT_INVALID=2, /* will be set if the certificate
+				* was not verified.
+				*/
+	GNUTLS_CERT_REVOKED=32, /* in X.509 this will be set only if CRLs are checked
+				 */
+
+	/* Those are extra information about the verification
+	 * process. Will be set only if the certificate was 
+	 * not verified.
+	 */
+	GNUTLS_CERT_SIGNER_NOT_FOUND=64,
+	GNUTLS_CERT_SIGNER_NOT_CA=128
 } gnutls_certificate_status;
 
 typedef enum gnutls_certificate_request { GNUTLS_CERT_IGNORE, GNUTLS_CERT_REQUEST=1, GNUTLS_CERT_REQUIRE } gnutls_certificate_request;
@@ -126,7 +131,7 @@ typedef enum gnutls_close_request { GNUTLS_SHUT_RDWR=0, GNUTLS_SHUT_WR=1 } gnutl
 
 typedef enum HandshakeState { STATE0=0, STATE1, STATE2, STATE3, STATE4, STATE5,
 	STATE6, STATE7, STATE8, STATE9, STATE20=20, STATE21,
-	STATE30=30, STATE31, STATE50=50, STATE60=60, STATE61 } HandshakeState;
+	STATE30=30, STATE31, STATE50=50, STATE60=60, STATE61, STATE62 } HandshakeState;
 
 typedef enum HandshakeType { GNUTLS_HELLO_REQUEST, GNUTLS_CLIENT_HELLO, GNUTLS_SERVER_HELLO,
 		     GNUTLS_CERTIFICATE_PKT=11, GNUTLS_SERVER_KEY_EXCHANGE,
@@ -134,27 +139,34 @@ typedef enum HandshakeType { GNUTLS_HELLO_REQUEST, GNUTLS_CLIENT_HELLO, GNUTLS_S
 		     GNUTLS_CERTIFICATE_VERIFY, GNUTLS_CLIENT_KEY_EXCHANGE,
 		     GNUTLS_FINISHED=20 } HandshakeType;
 
+typedef HandshakeType gnutls_handshake_description;
+
 typedef struct {
 	opaque * data;
 	unsigned int size;
 } gnutls_datum;
-typedef gnutls_datum gnutls_sdatum;
+
+typedef struct {
+	const opaque * data;
+	unsigned int size;
+} gnutls_const_datum;
 
 #include <gnutls_buffer.h>
 
 /* This is the maximum number of algorithms (ciphers or macs etc).
  * keep it synced with GNUTLS_MAX_ALGORITHM_NUM in gnutls.h
  */
-#define MAX_ALGOS 10
+#define MAX_ALGOS 16
 
 #define MAX_CIPHERSUITES 256
 
 /* STATE */
 
 typedef enum gnutls_cipher_algorithm { GNUTLS_CIPHER_NULL=1, 
-	GNUTLS_CIPHER_ARCFOUR_128, GNUTLS_CIPHER_3DES_CBC, GNUTLS_CIPHER_RIJNDAEL_128_CBC, 
-	GNUTLS_CIPHER_TWOFISH_128_CBC, GNUTLS_CIPHER_RIJNDAEL_256_CBC,
-	GNUTLS_CIPHER_ARCFOUR_40
+	GNUTLS_CIPHER_ARCFOUR_128, GNUTLS_CIPHER_3DES_CBC, GNUTLS_CIPHER_AES_128_CBC, 
+	GNUTLS_CIPHER_AES_256_CBC, GNUTLS_CIPHER_ARCFOUR_40, 
+	
+	GNUTLS_CIPHER_RC2_40_CBC=90, GNUTLS_CIPHER_DES_CBC
 } gnutls_cipher_algorithm;
 
 typedef enum gnutls_kx_algorithm { GNUTLS_KX_RSA=1, GNUTLS_KX_DHE_DSS, 
@@ -162,7 +174,10 @@ typedef enum gnutls_kx_algorithm { GNUTLS_KX_RSA=1, GNUTLS_KX_DHE_DSS,
 	GNUTLS_KX_RSA_EXPORT, GNUTLS_KX_SRP_RSA, GNUTLS_KX_SRP_DSS
 } gnutls_kx_algorithm;
 
-typedef enum gnutls_mac_algorithm { GNUTLS_MAC_NULL=1, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA } gnutls_mac_algorithm;
+typedef enum gnutls_mac_algorithm { GNUTLS_MAC_UNKNOWN=0, GNUTLS_MAC_NULL=1, 
+GNUTLS_MAC_MD5, GNUTLS_MAC_SHA, GNUTLS_MAC_RMD160
+} gnutls_mac_algorithm;
+typedef gnutls_mac_algorithm gnutls_digest_algorithm;
 
 typedef enum gnutls_compression_method { GNUTLS_COMP_NULL=1, GNUTLS_COMP_ZLIB,
 	GNUTLS_COMP_LZO
@@ -190,8 +205,8 @@ typedef enum ContentType { GNUTLS_CHANGE_CIPHER_SPEC=20, GNUTLS_ALERT,
 	GNUTLS_HANDSHAKE, GNUTLS_APPLICATION_DATA 
 } ContentType;
 
-typedef enum gnutls_x509_certificate_format { GNUTLS_X509_FMT_DER, 
-	GNUTLS_X509_FMT_PEM } gnutls_x509_certificate_format;
+typedef enum gnutls_x509_crt_fmt { GNUTLS_X509_FMT_DER, 
+	GNUTLS_X509_FMT_PEM } gnutls_x509_crt_fmt;
 
 typedef enum gnutls_pk_algorithm { GNUTLS_PK_RSA = 1, GNUTLS_PK_DSA,
 	GNUTLS_PK_UNKNOWN = 0xff
@@ -199,6 +214,7 @@ typedef enum gnutls_pk_algorithm { GNUTLS_PK_RSA = 1, GNUTLS_PK_DSA,
 
 /* STATE (stop) */
 
+typedef void (*LOG_FUNC)( int, const char*);
 
 /* Pull & Push functions defines: 
  */
@@ -212,8 +228,10 @@ typedef int (*gnutls_db_remove_func)(void*, gnutls_datum key);
 typedef gnutls_datum (*gnutls_db_retr_func)(void*, gnutls_datum key);
 
 typedef struct AUTH_CRED {
-	gnutls_kx_algorithm algorithm;
-	/* the type of credentials depends on algorithm */
+	gnutls_credentials_type algorithm;
+
+	/* the type of credentials depends on algorithm 
+	 */
 	void* credentials;
 	struct AUTH_CRED* next;
 } AUTH_CRED;
@@ -221,22 +239,22 @@ typedef struct AUTH_CRED {
 
 struct GNUTLS_KEY_INT {
 	/* For DH KX */
-	gnutls_sdatum			key;
-	MPI				KEY;
-	MPI				client_Y;
-	MPI				client_g;
-	MPI				client_p;
-	MPI				dh_secret;
+	gnutls_datum			key;
+	GNUTLS_MPI				KEY;
+	GNUTLS_MPI				client_Y;
+	GNUTLS_MPI				client_g;
+	GNUTLS_MPI				client_p;
+	GNUTLS_MPI				dh_secret;
 	/* for SRP */
-	MPI				A;
-	MPI				B;
-	MPI				u;
-	MPI				b;
-	MPI				a;
-	MPI				x;
+	GNUTLS_MPI				A;
+	GNUTLS_MPI				B;
+	GNUTLS_MPI				u;
+	GNUTLS_MPI				b;
+	GNUTLS_MPI				a;
+	GNUTLS_MPI				x;
 	/* RSA: e, m
 	 */
-	MPI 				rsa[2];
+	GNUTLS_MPI 				rsa[2];
 	
 	/* this is used to hold the peers authentication data 
 	 */
@@ -292,7 +310,7 @@ typedef enum gnutls_server_name_type { GNUTLS_NAME_DNS=1
 
 typedef struct {
    opaque name[MAX_SERVER_NAME_SIZE];
-   int name_length;
+   uint name_length;
    gnutls_server_name_type type;
 } server_name_st;
 
@@ -301,7 +319,7 @@ typedef struct {
 typedef struct {
 	server_name_st  server_names[MAX_SERVER_NAME_EXTENSIONS]; 
 	                /* limit server_name extensions */
-	int		server_names_size;
+	uint		server_names_size;
 	opaque 		srp_username[MAX_SRP_USERNAME];
 } TLSExtensions;
 
@@ -437,6 +455,12 @@ typedef struct {
 	int				may_write;
 
 	int				last_alert; /* last alert received */
+
+	/* The last handshake messages sent or received.
+	 */
+	int				last_handshake_in;
+	int				last_handshake_out;
+
 	/* this is the compression method we are going to use */
 	gnutls_compression_method		compression_method;
 	/* priorities */
@@ -559,7 +583,7 @@ typedef struct {
 	 */
 	uint16			extensions_sent[MAX_EXT_TYPES];
 	uint16			extensions_sent_size;
-
+	
 	/* is 0 if we are to send the whole PGP key, or non zero
 	 * if the fingerprint is to be sent.
 	 */
@@ -567,12 +591,10 @@ typedef struct {
 
 	/* This holds the default version that our first
 	 * record packet will have. */
-	gnutls_protocol_version		default_record_version;
+	opaque			default_record_version[2];
 
 	int			cbc_protection_hack;
 
-	int			rsa_pms_check; /* 0 means enabled */
-
 	void*			user_ptr;
 
 	int			enable_private;/* non zero to
@@ -589,6 +611,25 @@ typedef struct {
 	 * openpgp key. (if the peer sends a fingerprint)
 	 */
 	gnutls_openpgp_recv_key_func openpgp_recv_key_func;
+	
+	/* If non zero the server will not advertize the CA's he
+	 * trusts (do not send an RDN sequence).
+	 */
+	int			ignore_rdn_sequence;
+
+	/* This is used to set an arbitary version in the RSA
+	 * PMS secret. Can be used by clients to test whether the
+	 * server checks that version.
+	 */
+	opaque			rsa_pms_version[2];
+
+	char*			srp_username;
+	char*			srp_password;
+	
+	/* This is only set in SRP, when the handshake is
+	 * restarted if an username is not found.
+	 */
+	int			handshake_restarted;
 
 	/* If you add anything here, check _gnutls_handshake_internal_state_clear().
 	 */
@@ -605,25 +646,13 @@ struct gnutls_session_int {
 typedef struct gnutls_session_int *gnutls_session;
 
 typedef struct {
-	int bits;
-	MPI _prime;
-        MPI _generator;
-        gnutls_datum generator;
-       	gnutls_datum prime;
-        int local;              /* indicates if it is 
-                                 * not malloced, !=0 indicates malloced
-                                 */
+	GNUTLS_MPI _prime;
+        GNUTLS_MPI _generator;
 } _gnutls_dh_params;
 
 #define gnutls_dh_params _gnutls_dh_params*
 
-
-typedef struct {
-	int bits;
-	MPI params[RSA_PRIVATE_PARAMS];
-} _gnutls_rsa_params;
-
-#define gnutls_rsa_params _gnutls_rsa_params*
+#define gnutls_rsa_params gnutls_x509_privkey
 
 
 /* functions */
@@ -647,4 +676,7 @@ void _gnutls_free_auth_info( gnutls_session session);
 void _gnutls_set_adv_version( gnutls_session, gnutls_protocol_version);
 gnutls_protocol_version _gnutls_get_adv_version( gnutls_session);
 
+int gnutls_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, 
+	void* result, size_t* result_size);
+
 #endif /* GNUTLS_INT_H */
