@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000,2001 Nikos Mavroyanopoulos <nmav@hellug.gr>
+ *  Copyright (C) 2000,2001,2003 Nikos Mavroyanopoulos <nmav@hellug.gr>
  *
- * This file is part of GNUTLS.
+ *  This file is part of GNUTLS.
  *
  *  The GNUTLS library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public   
@@ -27,8 +27,9 @@
 #include <gnutls_datum.h>
 #include <x509_b64.h>
 
-static const uint8 b64table[64] =
+static const uint8 b64table[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
 static const uint8 asciitable[128] = {
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
@@ -54,7 +55,7 @@ static const uint8 asciitable[128] = {
 	0xff, 0xff
 };
 
-inline static int encode(uint8 * result, const uint8 * data, int left)
+inline static int encode(char * result, const uint8 * data, int left)
 {
 
 	int data_len;
@@ -102,7 +103,7 @@ inline static int encode(uint8 * result, const uint8 * data, int left)
  * result should be 3 bytes
  */
 #define TOASCII(c) (c < 127 ? asciitable[c] : 0xff)
-inline static int decode(uint8 * result, const uint8 * data)
+inline static int decode(uint8 * result, const opaque * data)
 {
 	uint8 a1, a2;
 	int ret = 3;
@@ -133,7 +134,7 @@ inline static int decode(uint8 * result, const uint8 * data)
 	return ret;
 }
 
-/* encodes data and puts the result into result (localy alocated)
+/* encodes data and puts the result into result (locally allocated)
  * The result_size is the return value
  */
 int _gnutls_base64_encode(const uint8 * data, size_t data_size, uint8 ** result)
@@ -142,13 +143,7 @@ int _gnutls_base64_encode(const uint8 * data, size_t data_size, uint8 ** result)
 	int ret, tmp;
 	char tmpres[4];
 
-	ret = data_size % 3;
-	if (ret != 0)
-		ret = 4;
-	else
-		ret = 0;
-
-	ret += (data_size / 3) * 4;
+	ret = B64SIZE( data_size);
 
 	(*result) = gnutls_malloc(ret + 1);
 	if ((*result) == NULL)
@@ -167,8 +162,16 @@ int _gnutls_base64_encode(const uint8 * data, size_t data_size, uint8 ** result)
 	return ret;
 }
 
-/* encodes data and puts the result into result (localy alocated)
- * The result_size is the return value
+#define INCR(what, size) \
+	what+=size; \
+	if (what > ret) { \
+		gnutls_assert(); \
+		gnutls_free( (*result)); *result = NULL; \
+		return GNUTLS_E_INTERNAL_ERROR; \
+	}
+
+/* encodes data and puts the result into result (locally allocated)
+ * The result_size (including the null terminator) is the return value.
  */
 int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 			   uint8 ** result)
@@ -178,14 +181,17 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 	uint8 *ptr;
 	uint8 top[80];
 	uint8 bottom[80];
-	int pos;
+	int pos, bytes, top_len, bottom_len;
+	size_t msglen = strlen(msg);
+
+	if (msglen > 50) {
+		gnutls_assert();
+		return GNUTLS_E_BASE64_ENCODING_ERROR;
+	}
 
 	memset(bottom, 0, sizeof(bottom));
 	memset(top, 0, sizeof(top));
 
-	if (strlen(msg) > 50)
-		return GNUTLS_E_MEMORY_ERROR;
-
 	strcat(top, "-----BEGIN "); /* Flawfinder: ignore */
 	strcat(top, msg); /* Flawfinder: ignore */
 	strcat(top, "-----"); /* Flawfinder: ignore */
@@ -194,60 +200,70 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
 	strcat(bottom, msg); /* Flawfinder: ignore */
 	strcat(bottom, "-----\n"); /* Flawfinder: ignore */
 
-	ret = data_size % 3;
-	if (ret != 0)
-		ret = 4;
-	else
-		ret = 0;
+	top_len = strlen(top);
+	bottom_len = strlen(bottom);
 
-	ret += strlen(top) + strlen(bottom);
-
-	tmp = (data_size / 3) * 4;
-	ret += (tmp / 64) + (tmp % 64 > 0 ? 1 : 0);	/* add new lines */
-	ret += tmp;
+	ret = B64FSIZE( msglen, data_size);
 
 	(*result) = gnutls_calloc(1, ret + 1);
-	if ((*result) == NULL)
+	if ((*result) == NULL) {
+		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
+	}
 
-	strcat(*result, top); /* Flawfinder: ignore */
-	pos = strlen(top);
+	bytes = pos = 0;
+	INCR( bytes, top_len);
+	pos = top_len;
+
+	strcpy(*result, top); /* Flawfinder: ignore */
 
 	for (i = j = 0; i < data_size; i += 3, j += 4) {
+
 		tmp = encode(tmpres, &data[i], data_size - i);
 		if (tmp == -1) {
-			gnutls_free( (*result));
-			return GNUTLS_E_MEMORY_ERROR;
+			gnutls_assert();
+			gnutls_free( (*result)); *result = NULL;
+			return GNUTLS_E_BASE64_ENCODING_ERROR;
 		}
+
+		INCR(bytes, 4);
 		ptr = &(*result)[j + pos];
 
 		if ((j) % 64 == 0) {
+			INCR(bytes, 1);
 			pos++;
 			*ptr++ = '\n';
 		}
 		*ptr++ = tmpres[0];
 
 		if ((j + 1) % 64 == 0) {
-			*ptr++ = '\n';
+			INCR(bytes, 1);
 			pos++;
+			*ptr++ = '\n';
 		}
 		*ptr++ = tmpres[1];
 
 		if ((j + 2) % 64 == 0) {
+			INCR(bytes, 1);
 			pos++;
 			*ptr++ = '\n';
 		}
 		*ptr++ = tmpres[2];
 
 		if ((j + 3) % 64 == 0) {
-			*ptr++ = '\n';
+			INCR(bytes, 1);
 			pos++;
+			*ptr++ = '\n';
 		}
 		*ptr++ = tmpres[3];
 	}
 
-	strcat(*result, bottom); /* Flawfinder: ignore */
-	return ret;
+	INCR( bytes, bottom_len);
+
+	memcpy( &(*result)[ bytes-bottom_len], bottom, bottom_len);
+	(*result)[ bytes] = 0;
+
+	return ret + 1;
 }
 
 /**
@@ -260,9 +276,13 @@ int _gnutls_fbase64_encode(const char *msg, const uint8 * data, int data_size,
   * This function will convert the given data to printable data, using the base64 
   * encoding. This is the encoding used in PEM messages. If the provided
   * buffer is not long enough GNUTLS_E_SHORT_MEMORY_BUFFER is returned.
+  *
+  * The output string will be null terminated, although the size will not include
+  * the terminating null.
   * 
   **/
-int gnutls_pem_base64_encode( const char* msg, const gnutls_datum *data, char* result, int* result_size) {
+int gnutls_pem_base64_encode( const char* msg, const gnutls_datum *data, char* result, 
+	size_t* result_size) {
 opaque* ret;
 int size;
 
@@ -270,14 +290,14 @@ int size;
 	if (size < 0)
 		return size;
 
-	if (result==NULL || *result_size < size) {
+	if (result==NULL || *result_size < (uint)size) {
 		gnutls_free(ret);
 		*result_size = size;
 		return GNUTLS_E_SHORT_MEMORY_BUFFER;
 	} else {
 		memcpy( result, ret, size);
 		gnutls_free(ret);
-		*result_size = size;
+		*result_size = size - 1;
 	}
 
 	return 0;
@@ -293,8 +313,7 @@ int size;
   * encoding. This is the encoding used in PEM messages. This function will
   * allocate the required memory to hold the encoded data.
   *
-  * You should use the function gnutls_free() to
-  * free the returned data.
+  * You should use gnutls_free() to free the returned data.
   * 
   **/
 int gnutls_pem_base64_encode_alloc( const char* msg, const gnutls_datum *data, 
@@ -303,49 +322,41 @@ int gnutls_pem_base64_encode_alloc( const char* msg, const gnutls_datum *data,
 opaque* ret;
 int size;
 
+	if (result==NULL) return GNUTLS_E_INVALID_REQUEST;
+
 	size = _gnutls_fbase64_encode( msg, data->data, data->size, &ret);
 	if (size < 0)
 		return size;
 
-	if (result==NULL) {
-		gnutls_free(ret);
-		return GNUTLS_E_INVALID_REQUEST;
-	} else {
-		result->data = ret;
-	        result->size = size;
-	}
-
+	result->data = ret;
+	result->size = size - 1;
 	return 0;
 }
 
 
-/* decodes data and puts the result into result (localy alocated)
+/* decodes data and puts the result into result (locally allocated)
  * The result_size is the return value
  */
 int _gnutls_base64_decode(const uint8 * data, size_t data_size, uint8 ** result)
 {
 	unsigned int i, j;
-	int ret, tmp;
+	int ret, tmp, est;
 	uint8 tmpres[3];
 
-	data_size /= 4;
-	data_size *= 4;
-
-	ret = data_size / 4 * 3;
-	(*result) = gnutls_malloc(ret+1);
+	est = ((data_size * 3) / 4) + 1;
+	(*result) = gnutls_malloc(est);
 	if ((*result) == NULL)
 		return GNUTLS_E_MEMORY_ERROR;
 
-	for (i = j = 0; i < data_size; i += 4) {
+	ret = 0;
+	for (i = j = 0; i < data_size; i += 4, j += 3) {
 		tmp = decode(tmpres, &data[i]);
 		if (tmp < 0) {
-			gnutls_free( *result);
+			gnutls_free( *result); *result = NULL;
 			return tmp;
 		}
 		memcpy(&(*result)[j], tmpres, tmp);
-		if (tmp < 3)
-			ret -= (3 - tmp);
-		j += 3;
+		ret += tmp;
 	}
 	return ret;
 }
@@ -359,7 +370,7 @@ inline static int cpydata(const uint8 * data, int data_size, uint8 ** result)
 
 	(*result) = gnutls_malloc(data_size);
 	if (*result == NULL)
-		return -1;
+		return GNUTLS_E_MEMORY_ERROR;
 
 	for (j = i = 0; i < data_size; i++) {
 		if (data[i] == '\n' || data[i] == '\r')
@@ -377,48 +388,46 @@ inline static int cpydata(const uint8 * data, int data_size, uint8 ** result)
  */
 #define ENDSTR "-----\n"
 #define ENDSTR2 "-----\r"
-int _gnutls_fbase64_decode( const opaque* header, const opaque * data, size_t data_size,
+int _gnutls_fbase64_decode( const char* header, const opaque * data, size_t data_size,
 			   uint8 ** result)
 {
 	int ret;
-	char top[] = "-----BEGIN ";
-	char bottom[] = "\n-----END ";
+	static const char top[] = "-----BEGIN ";
+	static const char bottom[] = "\n-----END ";
 	uint8 *rdata;
 	int rdata_size;
 	uint8 *kdata;
 	int kdata_size;
 	char pem_header[128];
 
-	if (header != NULL) {
-		if (strlen(header) > sizeof(pem_header) - sizeof(top) - 1) {
-			gnutls_assert();
-			return GNUTLS_E_MEMORY_ERROR;
-		}
-		strcpy( pem_header, top);
-		strcpy( pem_header, header);
-		rdata = strstr( data, pem_header);
-	} else {
-		rdata = strstr( data, top);
-	}
+	_gnutls_str_cpy( pem_header, sizeof(pem_header), top);
+	if (header != NULL)
+		_gnutls_str_cat( pem_header, sizeof(pem_header), header);
+
+	rdata = strnstr( data, pem_header, data_size);
 
 	if (rdata==NULL) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		_gnutls_debug_log( "Could not find '%s'\n", pem_header);
+		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
 
 	data_size -= (unsigned long int)rdata-(unsigned long int)data;
 
 	if (data_size < 4 + strlen(bottom)) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
 
-	kdata = strstr( rdata, ENDSTR);
-	if (kdata==NULL) kdata = strstr( rdata, ENDSTR2);
-	
+	kdata = strnstr( rdata, ENDSTR, data_size);
+	/* allow CR as well.
+	 */
+	if (kdata==NULL) kdata = strnstr( rdata, ENDSTR2, data_size);
+
 	if (kdata==NULL) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		_gnutls_x509_log( "Could not find '%s'\n", ENDSTR);
+		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
 	data_size -= strlen(ENDSTR);
 	data_size -= (unsigned long int)kdata-(unsigned long int)rdata;
@@ -427,10 +436,10 @@ int _gnutls_fbase64_decode( const opaque* header, const opaque * data, size_t da
 	
 	/* position is now after the ---BEGIN--- headers */
 
-	kdata = strstr( rdata, bottom);
+	kdata = strnstr( rdata, bottom, data_size);
 	if (kdata==NULL) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
 
 	/* position of kdata is before the ----END--- footer 
@@ -439,14 +448,20 @@ int _gnutls_fbase64_decode( const opaque* header, const opaque * data, size_t da
 
 	if (rdata_size < 4) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
 
 	kdata_size = cpydata(rdata, rdata_size, &kdata);
 
+	if (kdata_size < 0) {
+		gnutls_assert();
+		return kdata_size;
+	}
+
 	if (kdata_size < 4) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		gnutls_free(kdata);
+		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
 
 	if ((ret = _gnutls_base64_decode( kdata, kdata_size, result)) < 0) {
@@ -470,12 +485,11 @@ int _gnutls_fbase64_decode( const opaque* header, const opaque * data, size_t da
   * is non null this function will search for "-----BEGIN header" and decode
   * only this part. Otherwise it will decode the first PEM packet found.
   *
-  * Note that b64_data should be null terminated.
-  * 
   * Returns GNUTLS_E_SHORT_MEMORY_BUFFER if the buffer given is not long enough,
   * or 0 on success.
   **/
-int gnutls_pem_base64_decode( const char* header, const gnutls_datum *b64_data, char* result, int* result_size) 
+int gnutls_pem_base64_decode( const char* header, const gnutls_datum *b64_data, 
+	unsigned char* result, size_t* result_size) 
 {
 opaque* ret;
 int size;
@@ -484,7 +498,7 @@ int size;
 	if (size < 0)
 		return size;
 
-	if (result==NULL || *result_size < size) {
+	if (result==NULL || *result_size < (uint)size) {
 		gnutls_free(ret);
 		*result_size = size;
 		return GNUTLS_E_SHORT_MEMORY_BUFFER;
@@ -509,11 +523,8 @@ int size;
   * "-----BEGIN header" and decode only this part. Otherwise it will decode the 
   * first PEM packet found.
   *
-  * You should use the function gnutls_free() to
-  * free the returned data.
+  * You should use gnutls_free() to free the returned data.
   *
-  * Note that b64_data should be null terminated.
-  * 
   **/
 int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_data, 
    gnutls_datum* result) 
@@ -521,18 +532,13 @@ int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_
 opaque* ret;
 int size;
 
+	if (result==NULL) return GNUTLS_E_INVALID_REQUEST;
+
 	size = _gnutls_fbase64_decode( header, b64_data->data, b64_data->size, &ret);
 	if (size < 0)
 		return size;
 
-	if (result==NULL) {
-		gnutls_free(ret);
-		return GNUTLS_E_INVALID_REQUEST;
-	} else {
-		result->data = ret;
-		result->size = size;
-	}
-
+	result->data = ret;
+	result->size = size;
 	return 0;
 }
-
