@@ -1,6 +1,5 @@
 /*
  * Copyright (C) 2001 Nikos Mavroyanopoulos
- * Copyright (C) 2004 Free Software Foundation
  *
  * This file is part of GNUTLS.
  *
@@ -76,8 +75,6 @@ int i;
 		_gnutls_kx_algorithms[i].name = "SRP";
 		_gnutls_kx_algorithms[i].algorithm = GNUTLS_KX_SRP;
 		_gnutls_kx_algorithms[i].auth_struct = &srp_auth_struct;
-		_gnutls_kx_algorithms[i].needs_dh_params = 0;
-		_gnutls_kx_algorithms[i].needs_rsa_params = 0;
 		_gnutls_kx_algorithms[i+1].name = 0;
 	}
 	i++;
@@ -86,8 +83,6 @@ int i;
 		_gnutls_kx_algorithms[i].name = "SRP RSA";
 		_gnutls_kx_algorithms[i].algorithm = GNUTLS_KX_SRP_RSA;
 		_gnutls_kx_algorithms[i].auth_struct = &srp_rsa_auth_struct;
-		_gnutls_kx_algorithms[i].needs_dh_params = 0;
-		_gnutls_kx_algorithms[i].needs_rsa_params = 0;
 		_gnutls_kx_algorithms[i+1].name = 0;
 	}
 	i++;
@@ -96,8 +91,6 @@ int i;
 		_gnutls_kx_algorithms[i].name = "SRP DSS";
 		_gnutls_kx_algorithms[i].algorithm = GNUTLS_KX_SRP_DSS;
 		_gnutls_kx_algorithms[i].auth_struct = &srp_dss_auth_struct;
-		_gnutls_kx_algorithms[i].needs_dh_params = 0;
-		_gnutls_kx_algorithms[i].needs_rsa_params = 0;
 		_gnutls_kx_algorithms[i+1].name = 0;
 	
 		return 0; /* ok */
@@ -150,35 +143,21 @@ int i;
 }
 
 
-extern OPENPGP_KEY_CREATION_TIME_FUNC _E_gnutls_openpgp_get_raw_key_creation_time;
-extern OPENPGP_KEY_EXPIRATION_TIME_FUNC _E_gnutls_openpgp_get_raw_key_expiration_time;
+extern OPENPGP_KEY_CREATION_TIME_FUNC _E_gnutls_openpgp_extract_key_creation_time;
+extern OPENPGP_KEY_EXPIRATION_TIME_FUNC _E_gnutls_openpgp_extract_key_expiration_time;
 extern OPENPGP_VERIFY_KEY_FUNC _E_gnutls_openpgp_verify_key;
+extern OPENPGP_CERT2GNUTLS_CERT _E_gnutls_openpgp_cert2gnutls_cert;
 extern OPENPGP_FINGERPRINT _E_gnutls_openpgp_fingerprint;
 extern OPENPGP_KEY_REQUEST _E_gnutls_openpgp_request_key;
 
-extern OPENPGP_RAW_KEY_TO_GCERT _E_gnutls_openpgp_raw_key_to_gcert;
-extern OPENPGP_RAW_PRIVKEY_TO_GKEY _E_gnutls_openpgp_raw_privkey_to_gkey;
-
-extern OPENPGP_KEY_TO_GCERT _E_gnutls_openpgp_key_to_gcert;
-extern OPENPGP_PRIVKEY_TO_GKEY _E_gnutls_openpgp_privkey_to_gkey;
-extern OPENPGP_KEY_DEINIT _E_gnutls_openpgp_key_deinit;
-extern OPENPGP_PRIVKEY_DEINIT _E_gnutls_openpgp_privkey_deinit;
-
 static void _gnutls_add_openpgp_functions(void) {
 #ifdef HAVE_LIBOPENCDK
-	_E_gnutls_openpgp_verify_key = _gnutls_openpgp_verify_key;
-	_E_gnutls_openpgp_get_raw_key_expiration_time = _gnutls_openpgp_get_raw_key_expiration_time;
-	_E_gnutls_openpgp_get_raw_key_creation_time = _gnutls_openpgp_get_raw_key_creation_time;
-	_E_gnutls_openpgp_fingerprint = _gnutls_openpgp_fingerprint;
+	_E_gnutls_openpgp_verify_key = gnutls_openpgp_verify_key;
+	_E_gnutls_openpgp_extract_key_expiration_time = gnutls_openpgp_extract_key_expiration_time;
+	_E_gnutls_openpgp_extract_key_creation_time = gnutls_openpgp_extract_key_creation_time;
+	_E_gnutls_openpgp_fingerprint = gnutls_openpgp_fingerprint;
 	_E_gnutls_openpgp_request_key = _gnutls_openpgp_request_key;
-
-	_E_gnutls_openpgp_raw_key_to_gcert = _gnutls_openpgp_raw_key_to_gcert;
-	_E_gnutls_openpgp_raw_privkey_to_gkey = _gnutls_openpgp_raw_privkey_to_gkey;
-
-	_E_gnutls_openpgp_key_to_gcert = _gnutls_openpgp_key_to_gcert;
-	_E_gnutls_openpgp_privkey_to_gkey = _gnutls_openpgp_privkey_to_gkey;
-	_E_gnutls_openpgp_key_deinit = gnutls_openpgp_key_deinit;
-	_E_gnutls_openpgp_privkey_deinit = gnutls_openpgp_privkey_deinit;
+	_E_gnutls_openpgp_cert2gnutls_cert = _gnutls_openpgp_cert2gnutls_cert;
 #endif
 }
 
