@@ -8,66 +8,74 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 97 "cli.gaa"
+#line 109 "cli.gaa"
 	char *rest_args;
-#line 87 "cli.gaa"
+#line 99 "cli.gaa"
 	char *srp_passwd;
-#line 84 "cli.gaa"
+#line 96 "cli.gaa"
 	char *srp_username;
-#line 81 "cli.gaa"
+#line 93 "cli.gaa"
 	char *x509_certfile;
-#line 78 "cli.gaa"
+#line 90 "cli.gaa"
 	char *x509_keyfile;
-#line 75 "cli.gaa"
+#line 87 "cli.gaa"
 	char *pgp_certfile;
-#line 72 "cli.gaa"
+#line 84 "cli.gaa"
 	char *pgp_trustdb;
-#line 69 "cli.gaa"
+#line 81 "cli.gaa"
 	char *pgp_keyring;
-#line 66 "cli.gaa"
+#line 78 "cli.gaa"
 	char *pgp_keyfile;
-#line 63 "cli.gaa"
+#line 75 "cli.gaa"
+	char *x509_crlfile;
+#line 72 "cli.gaa"
 	char *x509_cafile;
-#line 60 "cli.gaa"
+#line 69 "cli.gaa"
 	char **ctype;
-#line 59 "cli.gaa"
+#line 68 "cli.gaa"
 	int nctype;
-#line 56 "cli.gaa"
+#line 65 "cli.gaa"
 	char **kx;
-#line 55 "cli.gaa"
+#line 64 "cli.gaa"
 	int nkx;
-#line 52 "cli.gaa"
+#line 61 "cli.gaa"
 	char **macs;
-#line 51 "cli.gaa"
+#line 60 "cli.gaa"
 	int nmacs;
-#line 48 "cli.gaa"
+#line 57 "cli.gaa"
 	char **comp;
-#line 47 "cli.gaa"
+#line 56 "cli.gaa"
 	int ncomp;
-#line 44 "cli.gaa"
+#line 53 "cli.gaa"
 	char **proto;
-#line 43 "cli.gaa"
+#line 52 "cli.gaa"
 	int nproto;
-#line 40 "cli.gaa"
+#line 49 "cli.gaa"
 	char **ciphers;
-#line 39 "cli.gaa"
+#line 48 "cli.gaa"
 	int nciphers;
-#line 35 "cli.gaa"
+#line 44 "cli.gaa"
 	int record_size;
-#line 32 "cli.gaa"
+#line 41 "cli.gaa"
 	int port;
-#line 29 "cli.gaa"
+#line 38 "cli.gaa"
+	int print_cert;
+#line 35 "cli.gaa"
 	int xml;
-#line 26 "cli.gaa"
+#line 32 "cli.gaa"
+	int disable_extensions;
+#line 29 "cli.gaa"
 	int fingerprint;
-#line 23 "cli.gaa"
+#line 26 "cli.gaa"
 	int fmtder;
-#line 20 "cli.gaa"
+#line 23 "cli.gaa"
 	int crlf;
-#line 17 "cli.gaa"
+#line 20 "cli.gaa"
 	int starttls;
-#line 14 "cli.gaa"
+#line 17 "cli.gaa"
 	int resume;
+#line 14 "cli.gaa"
+	int debug;
 
 #line 114 "gaa.skel"
 };
